/*
 * message_file.C
 *
 * Deals with a standard unix mail file, such as /var/spool/mail/...
 *
 * Vincent Mallet, 1997, 1999
 */

// $Id: message_file.C,v 1.3 1999/04/20 23:15:20 vmallet Exp $

/* ViMmail - a small mail reader
 * Copyright (C) 1997, 1999 Vincent Mallet - vmallet@enst.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdlib.h>
#include <string.h>

#include "VimDebug.H"

#include "message_file.H"
#include "message.H"


// on refait strchr...  zarb.
static char *xstrchr(char *s, int c)
{
  while (*s && *s != c)
    s++;

  if (*s)
    return s;
  else
    return NULL;
}



// Constructor
Message_File:: Message_File(char *filename) : file(NULL), nb_messages(0), size(0), _cursor(0)
{
  file_name = strdup(filename);

  rescan_messages();
}


// Destructor
Message_File:: ~Message_File()
{
  // DEBUG_out("~Message_File()");

  free(file_name);
  
  close_file();
  
  clear_messages();
}


// Free memory, remove all messages.
void Message_File:: clear_messages(void)
{
  for (int i = 0; i < nb_messages; i++)
    delete messages[i];

  nb_messages = 0;
  _cursor = 0;
}


// will return TRUE if file has been modified
// since last rescan
int Message_File:: need_a_rescan(void)
{
  struct stat buf;

  stat(file_name, &buf);

  return (buf.st_size > size ||
	  buf.st_atime < buf.st_mtime);
}


// run through the message file and get all headers. 
// Instanciate Message objects.
int Message_File:: rescan_messages(void)
{
  Message *msg;
  int curm;
  int nblines;
  long fpos, mpos;
  char *p;
  char hfrom[300], hsubj[300], hdate[300];
  char line[300];

  clear_messages();

  if (!open_file())
    return 1; 


  fseek(file, 0L, SEEK_SET);

  printf("Loading messages from file '%s'...\n", file_name);
  

  curm = 0;
  fpos = 0;
  mpos = 0; 
  nblines = 0;

  while (fgets(line, 300, file)) {
    if (*line) line[strlen(line) - 1] = '\0';
    if (strlen(line) > 5 && strncmp(line, "From ", 5) == 0) {      
      
      if (curm) {
	if (nb_messages == VM_MAXMESSAGES_PER_FILE) { // @@ fix that. this code sucks.
	  fprintf(stderr, "WARNING: maximum number of messages per file reached ! (%d)\n", VM_MAXMESSAGES_PER_FILE);
	  fprintf(stderr, "please see file vimmail.H and recompile. ViMmail IS NOW UNSTABLE.\n");
	  break;
	}
	msg = new Message(this, mpos, nb_messages + 1, hfrom,  hsubj,  hdate, nblines, fpos - mpos);
	messages[nb_messages++] = msg;
      }
      mpos = fpos;
      curm = 1;
      nblines = 0;

      p = xstrchr(line + 5, ' ');
      if (p) {
	strcpy(hdate, p + 1);
	*p = '\0';
	strcpy(hfrom, line + 5);
      }
      else {
	strcpy(hfrom, line + 5);
	strcpy(hdate, "");
      }

      strcpy(hsubj, "");
    }
    else if (curm && strlen(line) >= 6 && strncmp(line, "Date: ", 6) == 0) {
      strcpy(hdate, line + 6);
    }
    else if (curm && strlen(line) >= 9 && strncmp(line, "Subject: ", 9) == 0) {
      strcpy(hsubj, line + 9);
      while (*hsubj && hsubj[strlen(hsubj) - 1] == ' ')
	hsubj[strlen(hsubj) - 1] = '\0';
    }
    
    if (curm)
      nblines++;

    fpos = ftell(file);
  }

  if (curm) {
    if (nb_messages == VM_MAXMESSAGES_PER_FILE) { // @@ fix that. this code sucks.
      fprintf(stderr, "WARNING: maximum number of messages per file reached ! (%d)\n", VM_MAXMESSAGES_PER_FILE);
      fprintf(stderr, "please see file vimmail.H and recompile. ViMmail IS NOW UNSTABLE.\n");
    } else {
      msg = new Message(this, mpos, nb_messages + 1, hfrom, hsubj, hdate, nblines, fpos - mpos);
      messages[nb_messages++] = msg;
    }
  }

  size = ftell(file);

#ifdef DEBUG
  printf("Loaded %d messages!\n", nb_messages);
#endif
  
  
  close_file();
  
  return 0;  
} // rescan_messages()


// text-based display (not used anymore)
void Message_File:: show_headers()
{
  for (int i = 0; i < nb_messages; i++)
    messages[i]->display_header();
}







void Message_File:: prompt(void)
// prompt. Des restes de la gestion ascii
// ecrite en eiffel.
{
  int done; 
  int maxnum;
  char s[240];
  int i;

  maxnum = nb_messages;
  show_headers();

  done = 0;
  while (!done) {
    printf("Quel message? (0 quit, -1 liste headers) ==> ");
    fgets(s, 239, stdin);

    i = atoi(s);

    switch(i) {
    case -1:
      show_headers();
      break;

    case 0:
      done = 1;
      break;
      
    default:
      if (i > maxnum)
	printf("numero de message invalide! (%d)\n", i);
      else
	messages[i-1]->display_core();
    }
  }
  

} // prompt()


char * Message_File:: toString(void)
{
  static char * theend = "\n_____ End Of Text ______\n";  // et ca c'est pas laid?
  char *buf;

  buf = (char *) malloc(size + strlen(theend));

  if (NULL == buf) {
    fprintf(stderr, "toString error: not enough memory\n");
    return NULL;
  }
  
  fseek(file, 0L, SEEK_SET);

  if (1 != fread(buf, size, 1, file)) {
    fprintf(stderr, "toString error: read error\n");
  }

  buf[size] = '\0';
  strcat(buf, theend);

  return buf;
} // toString()




// Allez... un peu d'eiffel mouhahaha!


int Message_File:: empty(void)
{
  return !nb_messages;
}


void Message_File:: start(void)
{
  _cursor = 0;
}

void Message_File:: end(void)
{
  if (nb_messages)
    _cursor = nb_messages - 1;
}

int Message_File:: off(void)
{
  return _cursor >= nb_messages;
}


void Message_File:: forth(void)
{
  if (!off())      // devrait etre une assertion.
    _cursor++;
}


void Message_File:: set_current(int index)
{
  if (index >= 0 && index < nb_messages)
    _cursor = index;
  else
    _cursor = nb_messages;
}


Message & Message_File:: item(void)
{
  return * messages[_cursor];
}

Message & Message_File:: item(int index)
{
  // ASSERT ( index is valid );
  return * messages[index];
}



int Message_File:: cursor(void)
{ 
  return _cursor; 
}


// quelques trucs prives

int Message_File:: open_file(void)
{
  if (!file) 
    file = fopen(file_name, "rb");  

  return (file != NULL);
}



void Message_File:: close_file(void)
{
  if (file) {
    fclose(file);
    file = NULL;
  }
}



